//
//	CS575 Technical Project
//	Begurgitate
//	Spring 1999
//	Shayne White
//	for Professor Bergendahl
//
//	MainWindow.cpp 
//	The main window class implementation file
//	Derived from BWindow
//
//	Log:
//	Author		Date		Change
//	ssw			03/06/1999	initial rev.
//	ssw			04/03/1999	got new machine, added pct to statusbars
//							added menus
//	ssw			04/07/1999	added update button, used messaging to pass info from button to app
//

#include <Application.h>
#include "Begurgitate.h"
#include "MainWindow.h"
#include <Button.h>
#include <be/interface/StatusBar.h>
#include <stdio.h>
#include <string.h>

#define BUTTON_MSG 'btnm'
#define APP_NAME "Begurgitate V0.3"

const rgb_color backcolor  = {216,216,216,0};
const uint32 MSG_UPDT = 'updt';

MainWindow::MainWindow(BRect frame)
				: BWindow(frame, APP_NAME, B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	Lock();
	//get info object
	my_sysinfo = new Sysinfo();
		
	//set up menus
	SetupMenus();
	
	// set up a rectangle and instantiate a new view
	BRect aRect( Bounds() );
	
	aView = new BStringView(aRect, "MainView", "Begurgitate", B_FOLLOW_ALL, B_WILL_DRAW);

	// add view to window
	AddChild(aView);
	aView->SetFont(be_bold_font);
	aView->SetFontSize(14);
	aView->SetText(APP_NAME);
	aView->SetAlignment(B_ALIGN_CENTER);
	aView->SetViewColor(backcolor);
	
	
	// for the status bars
	BRect bRect(50, 70, 250, 50);
	BRect cRect(50, 30, 250, 50);
	BRect dRect(50, 110, 250, 50);
	BRect eRect(50, 150, 250, 50);
	BRect fRect(50, 190, 250, 50);
	
	
	// for each resource, add a status bar to the window
    sb = new BStatusBar(cRect,"sbPages","RAM Usage",rampct);
    aView->AddChild(sb);
    
    sb1 = new BStatusBar(bRect,"sbSems","Semaphore Usage",sempct);
    aView->AddChild(sb1);					 	
    
    sb2 = new BStatusBar(dRect,"sbPorts","Port Usage",portpct);
    aView->AddChild(sb2);

    sb3 = new BStatusBar(eRect,"sbThreads","Thread Usage",threadpct);
    aView->AddChild(sb3);

    sb4 = new BStatusBar(fRect,"sbTeams","Team Usage",teampct);
    aView->AddChild(sb4);
	
	//add a button
	BButton *btnGetInfo = new BButton(BRect(75, 225, 230, 10),"get_info","Update", new BMessage(MSG_UPDT));
	aView->AddChild(btnGetInfo);
	btnGetInfo->SetTarget(aView);
	btnGetInfo->Invoke();
	
	GetInfo();
	
	Unlock();
	
	
}

MainWindow::~MainWindow()
{
	delete my_sysinfo;
}

void MainWindow::GetInfo()
{
	Lock();

	my_sysinfo->RefreshSysData();
	my_sysinfo->ShowData();
	
	// for each resource, add a status bar to the window
	sprintf(rampct,"%3.1f", (((double)my_sysinfo->UsedPages() / (double)my_sysinfo->MaxPages()) * 100.0) );
    strcat(rampct,"%");
    sb->Reset();
    sb->SetMaxValue(my_sysinfo->MaxPages());
    sb->Update(my_sysinfo->UsedPages(),"RAM Usage",rampct);
    
    sprintf(sempct,"%3.1f", (((double)my_sysinfo->UsedSems() / (double)my_sysinfo->MaxSems()) * 100.0) );
    strcat(sempct,"%");
    sb1->Reset();
    sb1->SetMaxValue(my_sysinfo->MaxSems());
    sb1->Update(my_sysinfo->UsedSems(),"Semaphore Usage",sempct);						 	
    
    sprintf(portpct,"%3.1f", (((double)my_sysinfo->UsedPorts() / (double)my_sysinfo->MaxPorts()) * 100.0) );
    strcat(portpct,"%");
    sb2->Reset();
    sb2->SetMaxValue(my_sysinfo->MaxPorts());
    sb2->Update(my_sysinfo->UsedPorts(),"Port Usage",portpct);
    
    sprintf(threadpct,"%3.1f", (((double)my_sysinfo->UsedThreads() / (double)my_sysinfo->MaxThreads()) * 100.0) );
    strcat(threadpct,"%");
	sb3->Reset();
    sb3->SetMaxValue(my_sysinfo->MaxThreads());
    sb3->Update(my_sysinfo->UsedThreads(),"Thread Usage",threadpct);	

	sprintf(teampct,"%3.1f", (((double)my_sysinfo->UsedTeams() / (double)my_sysinfo->MaxTeams()) * 100.0) );
    strcat(teampct,"%");    
	sb4->Reset();
    sb4->SetMaxValue(my_sysinfo->MaxTeams());
    sb4->Update(my_sysinfo->UsedTeams(),"Team Usage",teampct);	
    
    Unlock();
	
}

bool MainWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

void MainWindow::SetupMenus() 
{ 
    BMenuBar* menuBar; 
    BMenu* aMenu; 
    BMenuItem* aItem;

    BRect dummyRect( 0, 0, 10, 10 ); 
    menuBar = new BMenuBar( dummyRect, "menubar" ); 
    
    aMenu = new BMenu( "File" ); 
        aItem = new BMenuItem( "About...", new BMessage( B_ABOUT_REQUESTED ), 0, 0 ); 
        aItem->SetTarget( (BHandler *)be_app ); 
        aMenu->AddItem( aItem ); 

        aMenu->AddSeparatorItem(); 

        aItem = new BMenuItem( "Quit", new BMessage( B_QUIT_REQUESTED ), 'Q', 0 ); 
        aMenu->AddItem( aItem ); 
    
    menuBar->AddItem( aMenu ); 
    AddChild( menuBar ); 
	menuFrame = menuBar->Frame();

}

void
MainWindow::MessageReceived (BMessage* message)
{
	switch (message->what) {
		case MSG_UPDT:
		// if the user hits the update button, capture sysinfo again
			GetInfo();
			break;
	default:
		inherited::MessageReceived(message);
	}
	
	return;
}
